@@ -108,9 +108,11 @@ main (int argc, char *argv[])
 	textdomain (GETTEXT_PACKAGE);
 
 	args = NULL;
-	context = g_option_context_new (_("\n\nAdd connect to server mount"));
+	context = g_option_context_new (N_("\n\nAdd connect to server mount"));
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
 
+	g_option_context_set_translation_domain(context, GETTEXT_PACKAGE);
+
 	program = gnome_program_init ("nautilus-connect-server", VERSION,
 				      LIBGNOMEUI_MODULE, argc, argv,
 				      GNOME_PARAM_GOPTION_CONTEXT, context,
