@@ -74,6 +74,27 @@ nautilus_navigation_window_set_throbber_active (NautilusNavigationWindow *window
 	}
 }
 
+static void
+toolbar_reconfigured_cb (GtkToolItem *item,
+			 NautilusThrobber *throbber)
+{
+	GtkToolbarStyle style;
+	GtkIconSize size;
+
+	style = gtk_tool_item_get_toolbar_style (item);
+
+	if (style == GTK_TOOLBAR_BOTH)
+	{
+		size = GTK_ICON_SIZE_DIALOG;
+	}
+	else
+	{
+		size = GTK_ICON_SIZE_LARGE_TOOLBAR;
+	}
+
+	nautilus_throbber_set_size (throbber, size);
+}
+
 void
 nautilus_navigation_window_activate_throbber (NautilusNavigationWindow *window)
 {
@@ -89,17 +110,20 @@ nautilus_navigation_window_activate_throbber (NautilusNavigationWindow *window)
 	gtk_tool_item_set_expand (item, TRUE);
 	gtk_toolbar_insert (GTK_TOOLBAR (window->details->toolbar),
 			    item, -1);
-	
+
 	throbber = nautilus_throbber_new ();
 	gtk_widget_show (GTK_WIDGET (throbber));
-	
+
 	item = gtk_tool_item_new ();
 	gtk_container_add (GTK_CONTAINER (item), throbber);
 	gtk_widget_show (GTK_WIDGET (item));
 	
+	g_signal_connect (item, "toolbar-reconfigured",
+			  G_CALLBACK (toolbar_reconfigured_cb), throbber);
+
 	gtk_toolbar_insert (GTK_TOOLBAR (window->details->toolbar),
 			    item, -1);
-	
+
 	window->details->throbber = throbber;
 }
 
@@ -163,10 +187,10 @@ nautilus_navigation_window_load_extension_toolbar_items (NautilusNavigationWindo
 	
 	merge_id = gtk_ui_manager_new_merge_id (ui_manager);
 	window->details->extensions_toolbar_merge_id = merge_id;
-	action_group = gtk_action_group_new ("ExtensionsMenuGroup");
+	action_group = gtk_action_group_new ("ExtensionsToolbarGroup");
 	window->details->extensions_toolbar_action_group = action_group;
 	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
-	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+	gtk_ui_manager_insert_action_group (ui_manager, action_group, -1);
 	g_object_unref (action_group); /* owned by ui manager */
 
 	items = get_extension_toolbar_items (window);
